package it.andynaz.log;

import java.util.*;
import java.text.*;
import it.andynaz.utils.Utils;

/**
 * Configurazione di un {@link Logger}.
 * 
 * Racchiude tutte le propriet&agrave; di una configurazione di un
 * {@link Logger}, come ad esempio il livello del log.
 * 
 * <p>La classe contiene anche una configurazione condivisa, usata dai
 * <tt>Logger</tt> privi di configurazione, gestita tramite i metodi statici
 * della classe.</p>
 * 
 * @author andynaz
 * @version 0.1 2012/08/20
 */
public class Config{

	/**
	 * Livello minimo dei log da registrare.
	 * 
	 * Il livello &egrave; compreso nei log.
	 * 
	 * @since 0.1
	 */
	private Level level;

	/**
	 * Indica se nel log deve essere inserita la data.
	 * 
	 * La data viene inserita se il valore &egrave; <tt>true</tt>, e viene
	 * inserita secondo quanto ritornato dal metodo {@link #getDate()}.
	 * 
	 * @since 0.1
	 */
	private boolean withDate;

	/**
	 * Configurazione condivisa.
	 * 
	 * Viene usata dai Logger privi di configurazione.
	 * 
	 * <p>Viene inizialmente creata con livello INFO e senza l'uso della data.
	 * Pu&ograve; essere cambiata attraverso il metodo <tt>init</tt> o invocando
	 * i suoi metodi <tt>setXYZ()</tt>.</p>
	 * 
	 * <p><b>Nota:</b> essendo condivisa, se la configurazione viene cambiata,
	 * tutti i Logger privi configurazione avranno le nuove impostazioni.</p>
	 * 
	 * @since 0.1
	 */
	private static Config conf; //renderla final??
	
	static{
		Config.conf = new Config();
	}
	
	/**
	 * Costruisce una configurazione con un livello e un'indicazione sull'uso
	 * della data.
	 * 
	 * @param level livello minimo dei log da registrare
	 * @param withDate <tt>true</tt> se nei log va inserita la data,
	 *   <tt>false</tt> altrimenti
	 * 
	 * @since 0.1
	 */
	public Config(Level level, boolean withDate){
		this.level = level;
		this.withDate = withDate;
	}
	
	/**
	 * Costruisce una configurazione "standard".
	 * 
	 * La configurazione "standard" &egrave; sempre con livello {@link
	 * Level#INFO} e senza l'uso della data.
	 * 
	 * @since 0.1
	 */
	public Config(){
		this(Level.INFO, false);
	}
	
	
	/**
	 * Restituisce il livello minimo dei log da registrare.
	 * 
	 * @return livello minimo dei log da registrare
	 * 
	 * @since 0.1
	 */
	public Level getLevel(){
		return this.level;
	}
	
	/**
	 * Imposta il livello minimo dei log da registrare.
	 * 
	 * @param level livello minimo
	 * 
	 * @since 0.1
	 */
	public void setLevel(Level level){
		this.level = level;
	}
	
	/**
	 * Indica se nel log va inserita la data.
	 * 
	 * @return <tt>true</tt> se la data deve essere inserita nei log,
	 *   <tt>false</tt> altrimenti
	 * 
	 * @since 0.1
	 */
	public boolean isWithDate(){
		return this.withDate;
	}
	
	/**
	 * Imposta l'uso della data nel log.
	 * 
	 * @param withDate <tt>true</tt> se la data deve essere inserita nei log,
	 *   <tt>false</tt> altrimenti
	 * 
	 * @since 0.1
	 */
	public void setWithDate(boolean withDate){
		this.withDate = withDate;
	}
	
	
	/**
	 * Inizializza la configurazione condivisa.
	 * 
	 * Il metodo controlla due propriet&agrave;, <tt>andyLog.Level</tt> per il
	 * livello del log e <tt>andyLog.WithDate</tt> per l'uso della data.
	 * 
	 * <p>Il metodo imposta gli attributi delle sole propriet&agrave; che trova
	 * (e che sono non vuote).</p>
	 * 
	 * <p>Nel caso del livello, il valore della propriet&agrave; deve
	 * necessariamente coincidere con il nome di uno dei livelli, ad esempio
	 * <pre>andyLog.Level=DEBUG</pre>
	 * Se il valore della propriet&agrave; <tt>andyLog.Level</tt> non coincide
	 * con nessun livello, viene usato il livello <tt>INFO</tt>.</p>
	 * 
	 * <p>Nel caso dell'uso della data, il valore viene impostato a <tt>true</tt>
	 * se il valore &egrave; uguale (ignorando il case dei caratteri) alla
	 * stringa <tt>true</tt>.</p>
	 * 
	 * @param props propriet&agrave; da leggere
	 * 
	 * @since 0.1
	 */
	public static void init(Properties props){
		try{
			String level = props.getProperty("andyLog.Level");
			if( !Utils.isEmpty(level) )
				Config.conf.setLevel(Level.valueOf(level));
		} catch(IllegalArgumentException e){
			Utils.printException("errore nell'inizializzazione della configurazione", e);
		}
		String withDate = props.getProperty("andyLog.WithDate");
		if( !Utils.isEmpty(withDate) )
			Config.conf.setWithDate(Boolean.parseBoolean(withDate));
	}
	
	/**
	 * Restituisce la configurazione condivisa.
	 * 
	 * Viene usata dai Logger privi di configurazione.
	 * 
	 * @return la configurazione condivisa
	 * 
	 * @since 0.1
	 */
	public static Config getConfig(){
		return Config.conf;
	}
	
	/**
	 * Resituisce la data odierna.
	 * 
	 * Il formato della data &egrave; <tt>AAAA-MM-GG HH:mm:ss</tt>.
	 * 
	 * @return una stringa con la data odierna.
	 * 
	 * @since 0.1
	 */
	protected String getDate(){
		SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
		return sdf.format(new Date());
	}
	
}
